
/*
 * EvolutionaryAlgorithmC.h -
 *
 */

#ifndef EvolutionaryAlgorithmC_h
#define EvolutionaryAlgorithmC_h 1


#include "SolutionPopulationC.h"
#include "ConstraintPopulationC.h"
#include "EncounterC.h"
#include "SelectionC.h"
#include "../StatisticsC.h"
#include "../Common/ArrayC.h"

class EvolutionaryAlgorithmC
{
	public:

		EvolutionaryAlgorithmC
		(
			PositiveIntegerT const iSizeOfSolutionPopulation,
			UnsignedIntegerT const iSizeOfOffspring,
			PositiveIntegerT const iSizeOfGene,
			PositiveIntegerT const iSizeOfFitnessHistory,
			BoolT const iStopConditionGeneration,
			UnsignedIntegerT const iMaximumNumberOfGenerations,
			BoolT const iStopConditionEncountersDone,
			UnsignedIntegerT const iMaximumNumberOfEncountersDone,
			BoolT const iStopConditionGeneratedGenotypes,
			UnsignedIntegerT const iMaximumNumberOfGeneratedGenotypes,
			UnsignedIntegerT const iNumberOfEncounters,
			RealT const iLinearRankingBias,
			ZeroToOneRealRealT const iNormalMutationProbability,
			ZeroToOneRealRealT const iEqualMutationProbability,
			StringT const iFitnessHistorySizeMethod,
			UnsignedIntegerT const iScreenUpdates,
			BoolT const iShowSolutionPopulationOnScreen,
			BoolT const iShowConstraintPopulationOnScreen,
			StatisticsC * iStatistics
		);

		~EvolutionaryAlgorithmC ();

		void const Run
		(
			RandomCspC * iProblem
		);

		BoolT const GetFoundSolution ();
		UnsignedIntegerT const GetNumberOfEncountersDone ();

	protected:

		BoolT const StopCondition () const;

		void const WriteIntermediateResults () const;

		void const WriteResults () const;

	private:

		typedef enum FitnessHistorySizeMethodE
		{
			none,
			fixed,
			populationsize,
			// NOT IMPLEMENTED // adaptive
		} FitnessHistorySizeMethodT;

		static StringT FitnessHistorySizeMethods [];

		UnsignedIntegerT SizeOfSolutionPopulation;
		UnsignedIntegerT SizeOfOffspring;
		UnsignedIntegerT SizeOfGene;

		FitnessHistorySizeMethodT FitnessHistorySizeMethod;
		UnsignedIntegerT SizeOfFitnessHistory;

		RealT LinearRankingBias;

		ZeroToOneRealRealT NormalMutationProbability;
		ZeroToOneRealRealT EqualMutationProbability;

		UnsignedIntegerT NumberOfEncounters;

		SolutionPopulationC * SolutionPopulation;
		ConstraintPopulationC * ConstraintPopulation;

		BoolT StopConditionGeneration;
		UnsignedIntegerT MaximumNumberOfGeneration;
		BoolT StopConditionEncountersDone;
		UnsignedIntegerT MaximumNumberOfEncountersDone;
		BoolT StopConditionGeneratedGenotypes;
		UnsignedIntegerT MaximumNumberOfGeneratedGenotypes;

		BoolT FoundSolution;
		UnsignedIntegerT NumberOfEncountersDone;

		UnsignedIntegerT ScreenUpdates;
		BoolT ShowSolutionPopulationOnScreen;
		BoolT ShowConstraintPopulationOnScreen;

		StatisticsC * Statistics;
		EncounterC * Encounter;
		RandomCspC * Problem;
		SelectionC * SolutionSelection;
		SelectionC * ConstraintSelection;

}; // EvolutionaryAlgorithmC

#endif

// eof EvolutionaryAlgorithmC.h
