
#include "PopulationPoolC.h"



template <class T>
PopulationPoolC<T>::PopulationPoolC
(
	UnsignedIntegerT const iMaximumSizeOfPopulation
)
{
	Bookmark = 0;
	CurrentPoolItem = 0;
	MaximumSizeOfPopulation = iMaximumSizeOfPopulation;
	SizeOfPopulation = 0;
	Pool = 0;

} // PopulationPoolC<T>::PopulationPoolC



template <class T>
PopulationPoolC<T>::~PopulationPoolC ()
{
	First ();
	while (!IsEnd ())
	{
		SetBookmark ();
		Next ();
		DeleteBookmark ();
	}

} // PopulationPoolC<T>::~PopulationPoolC



template <class T>
void const PopulationPoolC<T>::Insert
(
	T iItem
)
{
	PoolT temp;
	PoolT newrecord;

	temp = Pool;

	SizeOfPopulation++;

	if (SizeOfPopulation > MaximumSizeOfPopulation)
	{
		cerr << "Error: size of population pool has overgrown its maximum.\n";
		exit (1);
	}

	while ((temp != 0) && (temp->Genotype->GetFitnessValue () > iItem->GetFitnessValue ()))
	{
		temp = temp->Next;
	}

	newrecord = new PoolS;
	newrecord->Genotype = iItem;

	if (Pool == 0)
	{
		newrecord->Next = 0;
		newrecord->Previous = 0;
		Pool = newrecord;
		PoolEnd = Pool;
	}
	else
	{
		if (temp == 0)
		{
			newrecord->Previous = PoolEnd;
			newrecord->Previous->Next = newrecord;
			PoolEnd = newrecord;
		}
		else
		{
			if (temp->Previous != 0)
			{
				temp->Previous->Next = newrecord;
				newrecord->Previous = temp->Previous;
			}
			else
			{
				newrecord->Previous = 0;
				Pool = newrecord;
			}

			temp->Previous = newrecord;
		}

		newrecord->Next = temp;
	}

} // PopulationPoolC<T>::Insert



template <class T>
void const PopulationPoolC<T>::None ()
{
	CurrentPoolItem = 0;

} // PopulationPoolC<T>::None



template <class T>
void const PopulationPoolC<T>::First ()
{
	CurrentPoolItem = Pool;

} // PopulationPoolC<T>::First



template <class T>
void const PopulationPoolC<T>::Last ()
{
	CurrentPoolItem = PoolEnd;

} // PopulationPoolC<T>::Last



template <class T>
void const PopulationPoolC<T>::Next ()
{
	if (CurrentPoolItem == 0)
	{
		cerr << "Error: trying to move the current pointer in the populationpool beyond last entry.\n";
		exit (1);
	}
	else
	{
		CurrentPoolItem = CurrentPoolItem->Next;
	}

} // PopulationPoolC<T>::Next



template <class T>
void const PopulationPoolC<T>::Previous ()
{
	if (CurrentPoolItem == 0)
	{
		cerr << "Error: trying to move the current pointer in the populationpool beyond first entry.\n";
		exit (1);
	}
	else
	{
		CurrentPoolItem = CurrentPoolItem->Previous;
	}

} // PopulationPoolC<T>::Previous



template <class T>
BoolT const PopulationPoolC<T>::IsEnd () const
{
	return (CurrentPoolItem == 0);

} // PopulationPoolC<T>::IsEnd



template <class T>
T PopulationPoolC<T>::Current () const
{
	if (CurrentPoolItem != 0)
		return (CurrentPoolItem->Genotype);
	else
	{
		cerr << "Warning calling empty currentpointer.\n";
		return (0);
	}

} // PopulationPoolC<T>::Current



template <class T>
void const PopulationPoolC<T>::SetBookmark ()
{
	if (CurrentPoolItem)
		Bookmark = CurrentPoolItem;
	else
		cerr << "Warning: setting empty bookmark.\n";

} // PopulationPoolC<T>::SetBookmark



template <class T>
void const PopulationPoolC<T>::DeleteBookmark ()
{
	if (CurrentPoolItem == Bookmark)
	{
		cerr << "Error: cannot delete bookmark when it points to the same item as currentpointer." << endl;
		exit (1);
	}

	//delete Bookmark->Genotype;

	if (Bookmark->Previous != 0)
	{
		Bookmark->Previous->Next = Bookmark->Next;
	}
	else
	{
		Pool = Bookmark->Next;
	}

	if (Bookmark->Next != 0)
	{
		Bookmark->Next->Previous = Bookmark->Previous;
	}
	else
	{
		PoolEnd = Bookmark->Previous;
	}

	delete Bookmark;

	SizeOfPopulation--;
	Bookmark = 0;

} // PopulationPoolC<T>::DeleteBookmark


// eof PopulationPoolC.t
