
#ifndef SolutionGenotypeC_h
#define SolutionGenotypeC_h 1


#include "../Problem/randomcsp.h"
#include "../Common/FileC.h"
#include "../Common/ArrayC.h"
#include "../Types.h"
#include "FitnessC.h"
#include <iomanip.h>



typedef UnsignedIntegerT SolutionFitnessT;
typedef UnsignedIntegerT SolutionGeneT;
typedef UnsignedIntegerT SolutionGeneNumberT;

class EncounterC;

class SolutionGenotypeC 
{
	public:

		SolutionGenotypeC
		(
			UnsignedIntegerT iSizeOfGene,
			UnsignedIntegerT iSizeOfFitnessHistory,
			RandomCspC * iProblem
		);

		SolutionGenotypeC
		(
			const SolutionGenotypeC & iGenotpye
		);

		~SolutionGenotypeC ();

		SolutionGenotypeC const & operator=
		(
			SolutionGenotypeC const & iSolutionGenotype
		);

		BoolT const operator==
		(
			SolutionGenotypeC const & iSolutionGenotype
		) const;

		void const Print () const;

		void const Print
		(
			FileC & iFile
		) const;

		SolutionFitnessT const GetFitnessValue () const;

		UnsignedIntegerT const GetNumberOfUnsatisfiedConstraints ();

		SolutionGeneNumberT const GetSizeOfGene () const;

		SolutionGeneT const GetGene
		(
			SolutionGeneNumberT iGeneNumber
		) const;

		void const Mutate
		(
			RealT MutationProbability
		);

		void const Crossover
		(
			SolutionGenotypeC const & iOtherParent,
			SolutionGenotypeC & oChild1,
			SolutionGenotypeC & oChild2
		);

		void const UpdateFitness
		(
			BoolT iReward
		);

		void const InitialiseFitness
		(
			EncounterC * iEncounter
		);

	protected:

		void const SetGene
		(
			SolutionGeneNumberT iGeneNumber,
			SolutionGeneT iGene
		);

	private:

		UnsignedIntegerT Id;

		FitnessC <UnsignedIntegerT> * Fitness;

		BoolT Dirty;
		UnsignedIntegerT NumberOfUnsatisfiedConstraints;

		ArrayC <SolutionGeneT> * Data;
		UnsignedIntegerT SizeOfGene;

		RandomCspC * Problem;

}; // SolutionGenotypeC


#endif

// eof SolutionGenotypeC.h
