#ifndef SolutionPopulationC_h
#define SolutionPopulationC_h 1

#include "SolutionGenotypeC.h"
#include "PopulationPoolC.h"
#include "SelectionC.h"
#include "../Common/ArrayC.h"
#include <MLCG.h>


class SolutionPopulationC 
{
	public:
		
		SolutionPopulationC
		(
			UnsignedIntegerT const iSizeOfSolutionPopulation,
			UnsignedIntegerT const iSizeOfOffspring,
			UnsignedIntegerT const iSizeOfGene,
			UnsignedIntegerT const iSizeOfFitnessHistory,
			ZeroToOneRealRealT const iNormalMutationProbability,
			ZeroToOneRealRealT const iEqualMutationProbability,
			SelectionC * iSelection,
			RandomCspC * iProblem
		);

		~SolutionPopulationC ();

		UnsignedIntegerT const GetGenerationNumber () const;
		SolutionFitnessT const GetFitnessOfBestGenotype ();
		UnsignedIntegerT const GetNumberOfUnsatisfiedConstraintsOfBestGenotype ();

		void const Print ();
		void const PrintBestGenotype ();

		void const InitialiseFitness
		(
			EncounterC * iEncounter
		);

		void const DoOneGeneration ();

		void const InjectGenotype
		(
			SolutionGenotypeC * iSolutionGenotype
		);

		SolutionGenotypeC * FetchGenotype ();

		SolutionGenotypeC * GetFoundSolutionGenotype ();
		UnsignedIntegerT const GetNumberOfGeneratedGenotypes () const;
		UnsignedIntegerT const GetNumberOfGenerations () const;

		BoolT const GetFoundSolution () const;
		UnsignedIntegerT const GetNumberOfEncountersDone () const;

	protected:

		void const AddOffspring ();
		void const SelectParents ();
		void const Reproduce ();

	private:

		UnsignedIntegerT SizeOfSolutionPopulation;
		UnsignedIntegerT SizeOfOffspring;
		UnsignedIntegerT SizeOfGene;
		UnsignedIntegerT SizeOfFitnessHistory;

		ZeroToOneRealRealT EqualMutationProbability;
		ZeroToOneRealRealT NormalMutationProbability;

		PopulationPoolC <SolutionGenotypeC *> * Pool;
		ArrayC <SolutionGenotypeC *> * Offspring;
		ArrayC <SolutionGenotypeC *> * Parents;

		SolutionGenotypeC * FoundSolutionGenotype;
		BoolT FoundSolution;

		UnsignedIntegerT GenerationNumber;
		UnsignedIntegerT NumberOfGeneratedGenotypes;

		RandomCspC * Problem;
		EncounterC * Encounter;
		SelectionC * Selection;

};



#endif

// eof SolutionPopulationC.h
